from django.contrib import admin
from edc_consent.modeladmin_mixins import ModelAdminConsentMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_consent_admin
from ..forms import SubjectConsentForm
from ..models import SubjectConsent
from .modeladmin_mixins import SubjectConsentModelAdminMixin


@admin.register(SubjectConsent, site=meta_consent_admin)
class SubjectConsentAdmin(
    SubjectConsentModelAdminMixin,
    ModelAdminConsentMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = SubjectConsentForm



from edc_crf.model_mixins import CrfModelMixin as BaseCrfModelMixin
from edc_crf.model_mixins import CrfStatusModelMixin


class CrfModelMixin(CrfStatusModelMixin, BaseCrfModelMixin):
    class Meta(BaseCrfModelMixin.Meta):
        abstract = True



# Generated by Django 3.2.13 on 2022-09-29 14:42

from django.db import migrations
import edc_visit_tracking.managers


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0154_auto_20220928_0419"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="bloodresultsfbc",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsglu",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultshba1c",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsins",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultslft",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultslipid",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsrft",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="complications",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="complicationsglycemia",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="concomitantmedication",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="delivery",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="dietandlifestyle",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="egfrdropnotification",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="eq5d3l",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="followupexamination",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="followupvitals",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="glucose",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomics",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicssimple",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="hepatitistest",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="malariatest",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="medicationadherence",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="mnsi",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="otherarvregimens",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="patienthistory",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="physicalexam",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="pregnancyupdate",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="sf12",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="studymedication",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="subjectvisitmissed",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="urinedipsticktest",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="urinepregnancy",
            managers=[
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-12 01:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0025_auto_20210710_2247"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="BP > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="BP > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="BP > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="BP > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="BP > 180/120mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
    ]



# Generated by Django 5.2.3 on 2025-07-03 22:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0065_alter_historicaloffstudymedication_subject_identifier_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="subject_identifier",
            field=models.CharField(db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="subject_identifier",
            field=models.CharField(max_length=50, unique=True),
        ),
    ]



# Generated by Django 5.2.1 on 2025-06-02 18:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0062_alter_endofstudy_offstudy_reason_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicaloffstudymedication",
            name="singleton_field",
            field=models.CharField(
                db_index=True,
                editable=False,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="offstudymedication",
            name="singleton_field",
            field=models.CharField(
                editable=False,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                unique=True,
                verbose_name="subject identifier",
            ),
        ),
    ]



# Generated by Django 5.2.1 on 2025-06-02 18:43

from django.db import migrations
from django.db.models import F


def update_singeton_field(apps, schema_editor):
    model_cls = apps.get_model("meta_prn.offstudymedication")
    model_cls.objects.update(singleton_field=F("subject_identifier"))


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0063_historicaloffstudymedication_singleton_field_and_more"),
    ]

    operations = [migrations.RunPython(update_singeton_field)]



# Generated by Django 5.0.8 on 2024-08-22 19:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0030_auto_20240822_1637"),
    ]

    operations = [
        migrations.CreateModel(
            name="EndpointsProxy",
            fields=[],
            options={
                "verbose_name": "Endpoints (DM): All",
                "verbose_name_plural": "Endpoints (DM): All",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("meta_reports.endpoints",),
        ),
    ]



import contextlib

from edc_lab_panel.constants import FBC, LFT, LIPIDS, RFT
from edc_lab_panel.panels import fbc_panel, lft_panel, lipids_panel, rft_panel
from edc_qareports.sql_generator import CrfCase, RequisitionCase

qa_cases: list[RequisitionCase | CrfCase] = [
    RequisitionCase(
        label="FBC requisitioned but not entered",
        dbtable="meta_subject_bloodresultsfbc",
        label_lower="meta_subject.bloodresultsfbc",
        panel=FBC,
    ),
    RequisitionCase(
        label="RFT requisitioned but not entered",
        dbtable="meta_subject_bloodresultsrft",
        label_lower="meta_subject.bloodresultsrft",
        panel=RFT,
    ),
    RequisitionCase(
        label="LFT requisitioned but not entered",
        dbtable="meta_subject_bloodresultslft",
        label_lower="meta_subject.bloodresultslft",
        panel=LFT,
    ),
    RequisitionCase(
        label="LIPIDS requisitioned but not entered",
        dbtable="meta_subject_bloodresultslipids",
        label_lower="meta_subject.bloodresultslipids",
        panel=LIPIDS,
    ),
]

panels = {FBC: fbc_panel, RFT: rft_panel, LFT: lft_panel, LIPIDS: lipids_panel}
for abbrev, panel in panels.items():
    for utest_id in panel.utest_ids:
        with contextlib.suppress(ValueError):
            utest_id, _ = utest_id  # noqa: PLW2901
        qa_cases.append(
            CrfCase(
                label=f"{abbrev.upper()}: missing {utest_id} value/units",
                dbtable=f"meta_subject_bloodresults{abbrev.lower()}",
                label_lower=f"meta_subject.bloodresults{abbrev.lower()}",
                where=f"crf.{utest_id}_value is null or crf.{utest_id}_units is null",
            )
        )

    (
        CrfCase(
            label="No UREA value",
            dbtable="meta_subject_bloodresultsrft",
            label_lower="meta_subject.bloodresultsrft",
            where="crf.urea_value is null or crf.urea_units is null",
        ),
    )



# Generated by Django 5.2.7 on 2025-10-09 21:31

import _socket
import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.models.fields.other_charfield
import edc_model.validators.date
import edc_protocol.validators
import edc_visit_tracking.managers
import simple_history.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_lists",
            "0020_alter_abnormalfootappearanceobservations_extra_value_and_more",
        ),
        ("meta_subject", "0234_remove_spfq_site_remove_spfq_subject_visit_and_more"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="glucosefbg",
            name="endpoint_today",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "No. A repeat FBG will be scheduled"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Response is applicable if reporting after 24/11/2023 and both the FBG and OGTT were performed",
                max_length=25,
                verbose_name="Has the participant reached a study endpoint today?",
            ),
        ),
        migrations.AddField(
            model_name="historicalglucosefbg",
            name="endpoint_today",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("PENDING", "No. A repeat FBG will be scheduled"),
                    ("No", "No"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Response is applicable if reporting after 24/11/2023 and both the FBG and OGTT were performed",
                max_length=25,
                verbose_name="Has the participant reached a study endpoint today?",
            ),
        ),
        migrations.CreateModel(
            name="HistoricalHivExitReview",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        default="",
                        editable=False,
                        help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                        max_length=75,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        default="",
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        default="",
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        default="",
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=10,
                        verbose_name="Device created",
                    ),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=10,
                        verbose_name="Device modified",
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(blank=True, default="", max_length=50),
                ),
                (
                    "consent_version",
                    models.CharField(blank=True, default="", max_length=10),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="for example, why some data is still pending",
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "viral_load",
                    models.IntegerField(
                        blank=True,
                        help_text="copies/mL",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999999),
                        ],
                        verbose_name="Last viral load",
                    ),
                ),
                (
                    "viral_load_date",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date of last viral load"
                    ),
                ),
                (
                    "cd4",
                    models.IntegerField(
                        blank=True,
                        help_text="cells/mm<sup>3</sup>",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(3000),
                        ],
                        verbose_name="Last CD4",
                    ),
                ),
                (
                    "cd4_date",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date of last CD4"
                    ),
                ),
                (
                    "other_current_arv_regimen",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "current_arv_regimen_start_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="When did the patient start this current antiretroviral therapy regimen?",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "current_arv_regimen",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_lists.arvregimens",
                        verbose_name="Which antiretroviral therapy regimen is the patient currently on?",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical HIV Exit Review",
                "verbose_name_plural": "historical HIV Exit Review",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HivExitReview",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        default="",
                        editable=False,
                        help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                        max_length=75,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        default="",
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        default="",
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        default="",
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=10,
                        verbose_name="Device created",
                    ),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=10,
                        verbose_name="Device modified",
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(blank=True, default="", max_length=50),
                ),
                (
                    "consent_version",
                    models.CharField(blank=True, default="", max_length=10),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="for example, why some data is still pending",
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "viral_load",
                    models.IntegerField(
                        blank=True,
                        help_text="copies/mL",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(999999),
                        ],
                        verbose_name="Last viral load",
                    ),
                ),
                (
                    "viral_load_date",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date of last viral load"
                    ),
                ),
                (
                    "cd4",
                    models.IntegerField(
                        blank=True,
                        help_text="cells/mm<sup>3</sup>",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(3000),
                        ],
                        verbose_name="Last CD4",
                    ),
                ),
                (
                    "cd4_date",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date of last CD4"
                    ),
                ),
                (
                    "other_current_arv_regimen",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "current_arv_regimen_start_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="When did the patient start this current antiretroviral therapy regimen?",
                    ),
                ),
                (
                    "current_arv_regimen",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="last_arv_regimen",
                        to="meta_lists.arvregimens",
                        verbose_name="Which antiretroviral therapy regimen is the patient currently on?",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "HIV Exit Review",
                "verbose_name_plural": "HIV Exit Review",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["subject_visit", "site"],
                        name="meta_subjec_subject_d9ce05_idx",
                    ),
                    models.Index(
                        fields=["subject_visit", "report_datetime"],
                        name="meta_subjec_subject_ae7af9_idx",
                    ),
                ],
            },
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
    ]



from django import forms
from edc_form_validators import FormValidator


class SubjectScreeningFormValidator(FormValidator):
    def clean(self):
        raise forms.ValidationError(
            "This form may only be completed one part at a time. "
            "See the EDC's `Screening` section."
        )



from django.contrib import admin
from django.template.loader import render_to_string

from ..admin_site import meta_reports_admin
from ..models import EndpointsProxy
from .modeladmin_mixins import EndpointsModelAdminMixin


@admin.register(EndpointsProxy, site=meta_reports_admin)
class EndpointsAllAdmin(EndpointsModelAdminMixin, admin.ModelAdmin):
    def rendered_change_list_note(self):
        return render_to_string("meta_reports/endpoints_all_change_list_note.html")



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin

from ...models import HealthEconomicsUpdate


class HealthEconomicsUpdateForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = None

    class Meta:
        model = HealthEconomicsUpdate
        fields = "__all__"



import sys
from decimal import Decimal

from django.core.exceptions import ObjectDoesNotExist
from django.core.management import BaseCommand
from edc_offstudy.exceptions import OffstudyError
from edc_pharmacy.exceptions import NextStudyMedicationError, StudyMedicationError
from edc_pharmacy.models import DosageGuideline, Formulation, Medication
from edc_pharmacy.refill.refill_creator import RefillCreatorError
from edc_visit_schedule.utils import is_baseline
from tqdm import tqdm

from meta_pharmacy.constants import METFORMIN
from meta_subject.models import StudyMedication


def create_missing_rx_refills():
    messages = []
    medication = Medication.objects.get(name=METFORMIN)
    formulation = Formulation.objects.get(medication=medication, strength=500)
    dosage_guideline_baseline = DosageGuideline.objects.get(
        medication=medication, dose=Decimal("1000.0")
    )
    dosage_guideline_followup = DosageGuideline.objects.get(
        medication=medication, dose=Decimal("2000.0")
    )
    qs = StudyMedication.objects.all()
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.refill_identifier = obj.id
        if not obj.formulation:
            obj.formulation = formulation
        if not obj.dosage_guideline:
            if is_baseline(obj.related_visit):
                obj.dosage_guideline = dosage_guideline_baseline
            else:
                obj.dosage_guideline = dosage_guideline_followup
        messages = save_or_raise(obj, messages=messages)
    print_messages(messages)


def save_or_raise(obj, messages: list[str] | None = None):
    try:
        obj.save()
    except NextStudyMedicationError as e:
        messages.append(f"NextStudyMedicationError: {e}")
    except ObjectDoesNotExist as e:
        messages.append(
            f"ObjectDoesNotExist: {obj.subject_identifier}, {obj.related_visit}, {e!s}"
        )
    except StudyMedicationError as e:
        messages.append(f"StudyMedicationError: {e}")
    except RefillCreatorError as e:
        messages.append(f"RefillCreatorError: {e}")
    except OffstudyError as e:
        messages.append(f"OffstudyError: {e}")
    return messages


def print_messages(messages: list[str]):
    sys.stdout.write("\n")
    for message in messages:
        sys.stdout.write("\n")
        sys.stdout.write(message)


class Command(BaseCommand):
    def handle(self, *args, **options):  # noqa: ARG002
        create_missing_rx_refills()



# Generated by Django 5.0.1 on 2024-02-12 22:37

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django.utils.timezone
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_lists", "0017_complications_dmmedications_dmtreatments_and_more"),
        ("meta_subject", "0163_alter_birthoutcomes_options_and_more"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="DmReferralFollowup",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "referral_date",
                    models.DateField(verbose_name="Date of referral to diabetes clinic"),
                ),
                (
                    "attended",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Did you attend the diabetes clinic following referral from the META Trial",
                    ),
                ),
                (
                    "not_attended_reason",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="If 'No', please provide a reason for not seeking further care or follow up?",
                    ),
                ),
                (
                    "facility_attended",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="If ‘Yes’, please give the name of the facility you attended",
                    ),
                ),
                (
                    "attended_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="What was the date you attended the health facility named above?",
                    ),
                ),
                (
                    "healthcare_workers_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'healthcare worker', please specify ...",
                    ),
                ),
                (
                    "investigations_performed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=25,
                        verbose_name="Were any further investigations conducted at this visit?",
                    ),
                ),
                (
                    "investigations_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'investigations', please specify ...",
                    ),
                ),
                (
                    "on_dm_medications",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Are you currently taking any drug therapy for diabetes?",
                    ),
                ),
                (
                    "dm_medications_init_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="If ‘Yes’, please give the date when drug treatment was started.",
                    ),
                ),
                (
                    "dm_medications_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'drug treatments', please specify ...",
                    ),
                ),
                (
                    "medications_adherent",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Are you taking the diabetes drug treatments regularly, i.e. every day?",
                    ),
                ),
                (
                    "visual_score_slider",
                    models.CharField(help_text="%", max_length=3, verbose_name="Visual score"),
                ),
                (
                    "visual_score_confirmed",
                    models.IntegerField(
                        help_text="%",
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(100),
                        ],
                        verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
                    ),
                ),
                (
                    "last_missed_pill",
                    models.CharField(
                        choices=[
                            ("today", "today"),
                            ("yesterday", "yesterday"),
                            ("earlier_this_week", "earlier this week"),
                            ("last_week", "last week"),
                            ("lt_month_ago", "less than a month ago"),
                            ("gt_month_ago", "more than a month ago"),
                            ("NEVER", "have never missed taking my study pills"),
                        ],
                        max_length=25,
                        verbose_name="When was the last time you missed your study pill?",
                    ),
                ),
                (
                    "complications_checks",
                    models.ManyToManyField(
                        blank=True,
                        to="meta_lists.complications",
                        verbose_name="Have any tests for diabetes related complications been done?",
                    ),
                ),
                (
                    "dm_medications",
                    models.ManyToManyField(
                        to="meta_lists.dmmedications",
                        verbose_name="If ‘Yes’, please indicate which diabetes drug treatments you are currently taking.",
                    ),
                ),
                (
                    "dm_treatments",
                    models.ManyToManyField(
                        to="meta_lists.dmtreatments",
                        verbose_name="What treatment was prescribed at this visit?",
                    ),
                ),
                (
                    "healthcare_workers",
                    models.ManyToManyField(blank=True, to="meta_lists.healthcareworkers"),
                ),
                (
                    "investigations",
                    models.ManyToManyField(
                        blank=True,
                        to="meta_lists.investigations",
                        verbose_name="If ‘Yes’, please indicate what investigations were conducted.",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Diabetes referral follow-up",
                "verbose_name_plural": "Diabetes referral follow-up",
            },
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalDmReferralFollowup",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(editable=False, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(editable=False, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "referral_date",
                    models.DateField(verbose_name="Date of referral to diabetes clinic"),
                ),
                (
                    "attended",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Did you attend the diabetes clinic following referral from the META Trial",
                    ),
                ),
                (
                    "not_attended_reason",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="If 'No', please provide a reason for not seeking further care or follow up?",
                    ),
                ),
                (
                    "facility_attended",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="If ‘Yes’, please give the name of the facility you attended",
                    ),
                ),
                (
                    "attended_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="What was the date you attended the health facility named above?",
                    ),
                ),
                (
                    "healthcare_workers_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'healthcare worker', please specify ...",
                    ),
                ),
                (
                    "investigations_performed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=25,
                        verbose_name="Were any further investigations conducted at this visit?",
                    ),
                ),
                (
                    "investigations_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'investigations', please specify ...",
                    ),
                ),
                (
                    "on_dm_medications",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Are you currently taking any drug therapy for diabetes?",
                    ),
                ),
                (
                    "dm_medications_init_date",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="If ‘Yes’, please give the date when drug treatment was started.",
                    ),
                ),
                (
                    "dm_medications_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other 'drug treatments', please specify ...",
                    ),
                ),
                (
                    "medications_adherent",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=25,
                        verbose_name="Are you taking the diabetes drug treatments regularly, i.e. every day?",
                    ),
                ),
                (
                    "visual_score_slider",
                    models.CharField(help_text="%", max_length=3, verbose_name="Visual score"),
                ),
                (
                    "visual_score_confirmed",
                    models.IntegerField(
                        help_text="%",
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(100),
                        ],
                        verbose_name="<B><font color='orange'>Interviewer</font></B>: please transcribe the score indicated from above.",
                    ),
                ),
                (
                    "last_missed_pill",
                    models.CharField(
                        choices=[
                            ("today", "today"),
                            ("yesterday", "yesterday"),
                            ("earlier_this_week", "earlier this week"),
                            ("last_week", "last week"),
                            ("lt_month_ago", "less than a month ago"),
                            ("gt_month_ago", "more than a month ago"),
                            ("NEVER", "have never missed taking my study pills"),
                        ],
                        max_length=25,
                        verbose_name="When was the last time you missed your study pill?",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Diabetes referral follow-up",
                "verbose_name_plural": "historical Diabetes referral follow-up",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



# Generated by Django 5.2.7 on 2025-10-14 00:10

import edc_model.models.fields.other_charfield
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0235_glucosefbg_endpoint_today_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalhivexitreview",
            name="other_current_arv_regimen",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatienthistory",
            name="other_current_arv_regimen",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="hivexitreview",
            name="other_current_arv_regimen",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="patienthistory",
            name="other_current_arv_regimen",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                default="",
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



# Generated by Django 5.2.10 on 2026-01-23 13:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0021_diagnosticdevices"),
        ("meta_subject", "0240_glucose_diagnostic_device_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="glucose",
            old_name="diagnostic_device",
            new_name="fbg_diagnostic_device",
        ),
        migrations.RenameField(
            model_name="glucosefbg",
            old_name="diagnostic_device",
            new_name="fbg_diagnostic_device",
        ),
        migrations.RenameField(
            model_name="historicalglucose",
            old_name="diagnostic_device",
            new_name="fbg_diagnostic_device",
        ),
        migrations.RenameField(
            model_name="historicalglucosefbg",
            old_name="diagnostic_device",
            new_name="fbg_diagnostic_device",
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-14 03:27

import django.core.validators
import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0173_alter_glucosefbg_options_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="glucosefbg",
            name="glucose_abnormal",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="glucose_grade",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="glucose_grade_description",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="glucose_reportable",
        ),
        migrations.RemoveField(
            model_name="glucosefbg",
            name="is_poc",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="glucose_abnormal",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="glucose_grade",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="glucose_grade_description",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="glucose_reportable",
        ),
        migrations.RemoveField(
            model_name="historicalglucosefbg",
            name="is_poc",
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result date and time",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="glucose_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="FBG",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="assay_datetime",
            field=models.DateTimeField(
                blank=True,
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Result date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="glucose_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="FBG",
            ),
        ),
    ]



from django.db import models
from edc_constants.choices import YES_NO
from edc_model.models import BaseUuidModel
from edc_model.validators import date_not_future
from edc_protocol.validators import date_not_before_study_start

from ..model_mixins import CrfModelMixin


class OffStudyMedication(CrfModelMixin, BaseUuidModel):
    last_dose_date = models.DateField(
        verbose_name="Date of last known dose",
        validators=[date_not_before_study_start, date_not_future],
    )

    last_dose_date_estimated = models.CharField(
        verbose_name="Is this date estimated",
        max_length=15,
        choices=YES_NO,
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass



# Generated by Django 3.0.6 on 2020-05-30 15:01

import uuid

import _socket
import django.contrib.sites.managers
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django.utils.timezone
import django_revision.revision_field
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
        ("meta_lists", "0008_auto_20200528_1517"),
        ("edc_action_item", "0025_auto_20200528_0520"),
        ("meta_subject", "0044_auto_20200528_1853"),
    ]

    operations = [
        migrations.RenameModel(old_name="Followup", new_name="FollowupExamination"),
        migrations.RenameModel(
            old_name="HistoricalFollowup", new_name="HistoricalFollowupExamination"
        ),
    ]



from django.db import models
from django_db_views.db_view import DBView
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions

from .view_definition import get_view_definition


class UnattendedTwoInRow(QaReportModelMixin, DBView):
    appt_datetime = models.DateTimeField()

    first_value = models.CharField(verbose_name="First", max_length=25)

    second_value = models.CharField(verbose_name="Second", max_length=25)

    interval_days = models.IntegerField()

    from_now_days = models.IntegerField()

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "unattended_two_in_row_view"
        verbose_name = "R120: Unattended appointments: Two in a row"
        verbose_name_plural = "R120: Unattended appointments: Two in a row"
        default_permissions = qa_reports_permissions



from .health_economics import HealthEconomics
from .health_economics_simple import HealthEconomicsSimple
from .health_economics_update import HealthEconomicsUpdate

__all__ = ["HealthEconomics", "HealthEconomicsSimple", "HealthEconomicsUpdate"]



# Generated by Django 3.2.4 on 2021-07-02 01:58

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0056_auto_20210702_0426"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsglu",
            name="fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsglu",
            name="fasted_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="bloodresultsglu",
            name="fasted_duration_str",
            field=models.CharField(
                blank=True,
                help_text="Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="fasted",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Has the participant fasted?",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="fasted_duration_minutes",
            field=models.IntegerField(help_text="system calculated value", null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsglu",
            name="fasted_duration_str",
            field=models.CharField(
                blank=True,
                help_text="Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Expected something like 1h20m, 11h5m, etc",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-03 13:04

from django.db import migrations
import edc_model.models.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0038_alter_endofstudy_delivery_date_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicaloffstudymedication",
            name="reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="offstudymedication",
            name="reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



from .glucose_summary import GlucoseSummary
from .imp_substitutions import ImpSubstitutions
from .missing_screening_ogtt import NOTE_STATUSES, MissingOgttNote, MissingScreeningOgtt
from .on_study_missing_lab_values import OnStudyMissingLabValues
from .on_study_missing_values import OnStudyMissingValues
from .patient_history_missing_baseline_cd4 import PatientHistoryMissingBaselineCd4
from .unattended_three_in_row import UnattendedThreeInRow
from .unattended_three_in_row2 import UnattendedThreeInRow2
from .unattended_two_in_row import UnattendedTwoInRow

__all__ = [
    "NOTE_STATUSES",
    "GlucoseSummary",
    "ImpSubstitutions",
    "MissingOgttNote",
    "MissingScreeningOgtt",
    "OnStudyMissingLabValues",
    "OnStudyMissingValues",
    "PatientHistoryMissingBaselineCd4",
    "UnattendedThreeInRow",
    "UnattendedThreeInRow2",
    "UnattendedTwoInRow",
]



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
        select subject_identifier, site_id, appt_datetime, `first_value`,
        `second_value`, `third_value`,
        datediff(`third_date`, `first_date`) as `interval_days`,
        datediff(now(), `first_date`) as `from_now_days`
        from (
            select subject_identifier, site_id, appt_datetime,
            FIRST_VALUE(visit_code) OVER w as `first_value`,
            NTH_VALUE(visit_code, 2) OVER w as `second_value`,
            NTH_VALUE(visit_code, 3) OVER w as `third_value`,
            FIRST_VALUE(appt_datetime) OVER w as `first_date`,
            NTH_VALUE(appt_datetime, 3) OVER w as `third_date`
            from edc_appointment_appointment where visit_code_sequence=0 and appt_status="New"
            and appt_datetime <= now()
            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)
        ) as B
        where `second_value` is not null and `third_value` is not null
        """  # noqa
    sql_view = SqlViewGenerator(
        report_model="meta_reports.unattendedthreeinrow",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 6.0 on 2025-01-28 13:32

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django.utils.timezone
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_appointment", "0048_alter_appointment_site_and_more"),
        ("edc_facility", "0014_healthfacility_title_historicalhealthfacility_title"),
        ("edc_visit_schedule", "0017_alter_onschedule_managers"),
        ("meta_subject", "0215_alter_historicalstudymedication_stock_codes_and_more"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalNextAppointment",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(blank=True, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "appt_date",
                    models.DateField(
                        help_text="Should fall on an valid clinic day for this facility",
                        null=True,
                        verbose_name="Next scheduled routine/facility appointment",
                    ),
                ),
                (
                    "appt_datetime",
                    models.DateField(
                        help_text="Should fall on an valid clinic day for this facility",
                        null=True,
                        verbose_name="Next scheduled routine/facility appointment date and time",
                    ),
                ),
                (
                    "allow_create_interim",
                    models.BooleanField(
                        default=False,
                        help_text="Override date check to allow the EDC to create an interim appointment if the date is within window period of the current appointment.",
                        verbose_name="Override date check?",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "health_facility",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_facility.healthfacility",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "info_source",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        max_length=15,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_appointment.infosources",
                        verbose_name="What is the source of this appointment date",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="meta_subject.subjectvisit",
                    ),
                ),
                (
                    "visitschedule",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        help_text="Click SAVE to let the EDC suggest. Once selected, interim appointments will be flagged as not required/missed.",
                        max_length=15,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_visit_schedule.visitschedule",
                        verbose_name="Which study visit code is closest to this appointment date",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Next Appointment",
                "verbose_name_plural": "historical Next Appointments",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="NextAppointment",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "consent_model",
                    models.CharField(blank=True, max_length=50, null=True),
                ),
                (
                    "consent_version",
                    models.CharField(blank=True, max_length=10, null=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="If reporting today, use today's date/time, otherwise use the date/time this information was reported.",
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Report Date",
                    ),
                ),
                (
                    "appt_date",
                    models.DateField(
                        help_text="Should fall on an valid clinic day for this facility",
                        null=True,
                        verbose_name="Next scheduled routine/facility appointment",
                    ),
                ),
                (
                    "appt_datetime",
                    models.DateField(
                        help_text="Should fall on an valid clinic day for this facility",
                        null=True,
                        verbose_name="Next scheduled routine/facility appointment date and time",
                    ),
                ),
                (
                    "allow_create_interim",
                    models.BooleanField(
                        default=False,
                        help_text="Override date check to allow the EDC to create an interim appointment if the date is within window period of the current appointment.",
                        verbose_name="Override date check?",
                    ),
                ),
                (
                    "crf_status",
                    models.CharField(
                        choices=[
                            ("INCOMPLETE", "Incomplete (some data pending)"),
                            ("COMPLETE", "Complete"),
                        ],
                        default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                        help_text="If some data is still pending, flag this CRF as incomplete",
                        max_length=25,
                        verbose_name="CRF status",
                    ),
                ),
                (
                    "crf_status_comments",
                    models.TextField(
                        blank=True,
                        help_text="for example, why some data is still pending",
                        null=True,
                        verbose_name="Any comments related to status of this CRF",
                    ),
                ),
                (
                    "health_facility",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_facility.healthfacility",
                    ),
                ),
                (
                    "info_source",
                    models.ForeignKey(
                        max_length=15,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_appointment.infosources",
                        verbose_name="What is the source of this appointment date",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
                (
                    "subject_visit",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_subject.subjectvisit",
                    ),
                ),
                (
                    "visitschedule",
                    models.ForeignKey(
                        help_text="Click SAVE to let the EDC suggest. Once selected, interim appointments will be flagged as not required/missed.",
                        max_length=15,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_visit_schedule.visitschedule",
                        verbose_name="Which study visit code is closest to this appointment date",
                    ),
                ),
            ],
            options={
                "verbose_name": "Next Appointment",
                "verbose_name_plural": "Next Appointments",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["subject_visit", "site"],
                        name="meta_subjec_subject_0d7238_idx",
                    ),
                    models.Index(
                        fields=["subject_visit", "report_datetime"],
                        name="meta_subjec_subject_208c45_idx",
                    ),
                    models.Index(
                        fields=["modified", "created"],
                        name="meta_subjec_modifie_0f17e9_idx",
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="meta_subjec_user_mo_c4e256_idx",
                    ),
                ],
            },
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin
from edc_sites.forms import SiteModelFormMixin

from ..form_validators import ScreeningPartOneFormValidator
from ..models import ScreeningPartOne
from .field_lists import part_one_fields


class ScreeningPartOneForm(
    AlreadyConsentedFormMixin, SiteModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    form_validator_cls = ScreeningPartOneFormValidator

    class Meta:
        model = ScreeningPartOne
        fields = part_one_fields

        labels = {  # noqa: RUF012
            "site": "Which study site is this?",
        }
        help_texts = {  # noqa: RUF012
            "site": "This question is asked to confirm you are logged in to the correct site.",
        }



from django import forms
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ..form_validators import SubjectConsentFormValidator
from ..models import SubjectConsentV1
from .subject_consent_form import help_texts, widgets


class SubjectConsentV1Form(
    SiteModelFormMixin, ConsentModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    form_validator_cls = SubjectConsentFormValidator

    def validate_gender_of_consent(self):
        return None

    def validate_guardian_and_dob(self):
        return None

    class Meta:
        model = SubjectConsentV1
        fields = "__all__"
        help_texts = help_texts
        widgets = widgets



from django.contrib import admin
from edc_qareports.modeladmin_mixins import OnStudyMissingValuesModelAdminMixin

from ....admin_site import meta_reports_admin
from ....models import OnStudyMissingLabValues


@admin.register(OnStudyMissingLabValues, site=meta_reports_admin)
class OnStudyMissingLabValuesAdmin(OnStudyMissingValuesModelAdminMixin, admin.ModelAdmin):
    include_note_column: bool = True



# Generated by Django 3.2.11 on 2022-07-19 18:34

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("edc_action_item", "0029_auto_20220704_1841"),
        ("meta_subject", "0124_auto_20220714_2303"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="EgfrNotification",
            new_name="EgfrDropNotification",
        ),
        migrations.RenameModel(
            old_name="HistoricalEgfrNotification",
            new_name="HistoricalEgfrDropNotification",
        ),
        migrations.AlterModelOptions(
            name="egfrdropnotification",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "eGFR Drop Notification",
                "verbose_name_plural": "eGFR Drop Notifications",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalegfrdropnotification",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical eGFR Drop Notification",
                "verbose_name_plural": "historical eGFR Drop Notifications",
            },
        ),
    ]



# Generated by Django 5.2.3 on 2025-07-10 12:05

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0224_bloodresultsfbc_abnormal_summary_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="followupvitals",
            name="waist_circumference",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                help_text="in centimeters",
                max_digits=5,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(50.0),
                    django.core.validators.MaxValueValidator(175.0),
                ],
                verbose_name="Waist circumference",
            ),
        ),
        migrations.AddField(
            model_name="historicalfollowupvitals",
            name="waist_circumference",
            field=models.DecimalField(
                blank=True,
                decimal_places=1,
                help_text="in centimeters",
                max_digits=5,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(50.0),
                    django.core.validators.MaxValueValidator(175.0),
                ],
                verbose_name="Waist circumference",
            ),
        ),
    ]



from clinicedc_constants import NO, NOT_APPLICABLE, YES
from edc_screening.fc import FC
from edc_screening.screening_eligibility import ScreeningEligibility


class EligibilityPartTwo(ScreeningEligibility):
    eligible_fld_name = "eligible_part_two"
    reasons_ineligible_fld_name = "reasons_ineligible_part_two"

    def __init__(self, **kwargs):
        self.acute_condition = None
        self.acute_metabolic_acidosis = None
        self.advised_to_fast = None
        self.alcoholism = None
        self.appt_datetime = None
        self.congestive_heart_failure = None
        self.has_dm = None
        self.on_dm_medication = None
        self.liver_disease = None
        self.metformin_sensitivity = None
        self.renal_function_condition = None
        self.tissue_hypoxia_condition = None
        super().__init__(**kwargs)

    def get_required_fields(self) -> dict[str, FC]:
        return {
            "acute_condition": FC(NO, "Acute condition"),
            "acute_metabolic_acidosis": FC(NO, "Acute metabolic acidosis"),
            "advised_to_fast": FC([YES, NOT_APPLICABLE], "Not advised to fast"),
            "alcoholism": FC(NO, "Alcoholism"),
            "congestive_heart_failure": FC(NO, "Congestive heart failure"),
            "liver_disease": FC(NO, "Liver disease"),
            "metformin_sensitivity": FC(NO, "Metformin sensitivity"),
            "renal_function_condition": FC(NO, "Renal function"),
            "tissue_hypoxia_condition": FC(NO, "Tissue hypoxia"),
            "appt_datetime": FC(ignore_if_missing=True),
            "has_dm": FC(NO, "Diabetes"),
            "on_dm_medication": FC(NO, "taking anti-diabetic medications"),
        }

    def set_eligible_model_field(self):
        setattr(self.model_obj, self.eligible_fld_name, self.eligible)



from .dbviews import (
    GlucoseSummaryAdmin,
    ImpSubstitutionsAdmin,
    MissingOgttNoteModelAdmin,
    MissingScreeningOgttAdmin,
    OnStudyMissingLabValuesAdmin,
    OnStudyMissingValuesAdmin,
    PatientHistoryMissingBaselineCd4Admin,
    UnattendedThreeInRow2Admin,
    UnattendedThreeInRowAdmin,
    UnattendedTwoInRowAdmin,
)
from .endpoints_admin import EndpointsAdmin
from .endpoints_all_admin import EndpointsAllAdmin
from .last_imp_refill_admin import LastImpRefillAdmin

__all__ = [
    "EndpointsAdmin",
    "EndpointsAllAdmin",
    "GlucoseSummaryAdmin",
    "ImpSubstitutionsAdmin",
    "LastImpRefillAdmin",
    "MissingOgttNoteModelAdmin",
    "MissingScreeningOgttAdmin",
    "OnStudyMissingLabValuesAdmin",
    "OnStudyMissingValuesAdmin",
    "PatientHistoryMissingBaselineCd4Admin",
    "UnattendedThreeInRow2Admin",
    "UnattendedThreeInRowAdmin",
    "UnattendedTwoInRowAdmin",
]



from django.contrib import admin
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_lab_panel.panels import hba1c_panel
from edc_lab_results.admin import BloodResultsModelAdminMixin
from edc_lab_results.fieldsets import BloodResultFieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ...admin_site import meta_subject_admin
from ...forms import BloodResultsHba1cForm
from ...models import BloodResultsHba1c
from ..modeladmin import CrfModelAdminMixin

# TODO: add is poc? YES, always


@admin.register(BloodResultsHba1c, site=meta_subject_admin)
class BloodResultsHba1cAdmin(
    BloodResultsModelAdminMixin, CrfModelAdminMixin, SimpleHistoryAdmin
):
    form = BloodResultsHba1cForm
    fieldsets = BloodResultFieldset(
        hba1c_panel,
        extra_fieldsets=[
            (-1, action_fieldset_tuple),
        ],
    ).fieldsets



# Generated by Django 5.2.6 on 2025-09-24 05:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0019_auto_20250128_0143"),
    ]

    operations = [
        migrations.AlterField(
            model_name="abnormalfootappearanceobservations",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="abnormalfootappearanceobservations",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="abnormalfootappearanceobservations",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="arvregimens",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="arvregimens",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="arvregimens",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="baselinesymptoms",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="baselinesymptoms",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="baselinesymptoms",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="complications",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="complications",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="complications",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="diabetessymptoms",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="diabetessymptoms",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="diabetessymptoms",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="dmmedications",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="dmmedications",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="dmmedications",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="dmtreatments",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="dmtreatments",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="dmtreatments",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="healthcareworkers",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="healthcareworkers",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="healthcareworkers",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="hypertensionmedications",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="hypertensionmedications",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="hypertensionmedications",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="investigations",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="investigations",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="investigations",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="missedreferralreasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="missedreferralreasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="missedreferralreasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="offstudyreasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="offstudyreasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="offstudyreasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="oiprophylaxis",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="oiprophylaxis",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="oiprophylaxis",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissedreasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissedreasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissedreasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="symptoms",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="symptoms",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="symptoms",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
        migrations.AlterField(
            model_name="transferreasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="transferreasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="transferreasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-30 15:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0045_auto_20200530_1801"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="followupexamination",
            name="meta_subjec_subject_8cd273_idx",
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="admitted_hospital",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="If YES, were they admitted to hospital?",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="attended_clinic_sae",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If YES, submit a <u>Serious Adverse Event</u> Form",
                max_length=25,
                verbose_name="Does the event constitute a <u>Serious Adverse Event</u>",
            ),
        ),
        migrations.AlterField(
            model_name="followupexamination",
            name="prescribed_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Was the participant prescribed any other medication at this clinic or hospital visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="admitted_hospital",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="If YES, were they admitted to hospital?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="attended_clinic_sae",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If YES, submit a <u>Serious Adverse Event</u> Form",
                max_length=25,
                verbose_name="Does the event constitute a <u>Serious Adverse Event</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="prescribed_medication",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Was the participant prescribed any other medication at this clinic or hospital visit?",
            ),
        ),
        migrations.AddIndex(
            model_name="followupexamination",
            index=models.Index(
                fields=["subject_visit", "site", "id"],
                name="meta_subjec_subject_aad749_idx",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-08-26 01:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0142_auto_20220826_0322"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="bloodresultsfbc",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultsglu",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultshba1c",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultsins",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultslft",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultslipid",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="bloodresultsrft",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="egfrdropnotification",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="followupexamination",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultsfbc",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultsglu",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultshba1c",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultsins",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultslft",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultslipid",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultsrft",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalegfrdropnotification",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalfollowupexamination",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectvisitmissed",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="historicalurinepregnancy",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="subjectvisitmissed",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="urinepregnancy",
            name="tracking_identifier",
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, null=True, unique=True),
        ),
    ]



from __future__ import annotations

from django import forms
from edc_appointment.form_validator_mixins import NextAppointmentCrfFormValidatorMixin
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import NextAppointment

__all__ = ["NextAppointmentForm"]


class NextAppointmentCrfFormValidator(
    NextAppointmentCrfFormValidatorMixin,
    CrfFormValidator,
):
    pass


class NextAppointmentForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = NextAppointmentCrfFormValidator

    class Meta:
        model = NextAppointment
        fields = "__all__"
        labels = {  # noqa: RUF012
            "appt_date": "Next scheduled appointment date",
            "visitschedule": "Next scheduled appointment",
        }
        help_texts = {  # noqa: RUF012
            "appt_date": (
                "Default recommended. If changed, should be within "
                "the window period of the next appointment"
            ),
            "visitschedule": "Default recommended. Can only be the next visit",
        }



from typing import Any

from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin
from edc_randomization.site_randomizers import site_randomizers

from meta_edc.meta_version import get_meta_version


class HomeView(EdcViewMixin, NavbarViewMixin, TemplateView):
    template_name = "meta_edc/home.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "home"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        randomizer_cls = site_randomizers.get(get_meta_version())
        edc_randomization_url_name = (
            "edc_randomization_admin:"
            f"{randomizer_cls.model_cls()._meta.label_lower.replace('.', '_')}_changelist"
        )
        kwargs.update(edc_randomization_url_name=edc_randomization_url_name)
        return super().get_context_data(**kwargs)



import sys

from django.core.management.base import BaseCommand, CommandError
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.models import Medication, Rx
from edc_pharmacy.prescribe import create_prescription
from tqdm import tqdm

from meta_consent.models import SubjectConsent
from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN


class Command(BaseCommand):
    help = "Create missing prescriptions"

    def handle(self, *args, **options):  # noqa: ARG002
        medication = Medication.objects.get(name=METFORMIN)
        subject_identifiers = Rx.objects.values_list("subject_identifier", flat=True).all()
        subject_consents_wo_rx = SubjectConsent.objects.exclude(
            subject_identifier__in=subject_identifiers
        )
        total = subject_consents_wo_rx.count()
        for instance in tqdm(subject_consents_wo_rx, total=total):
            try:
                create_prescription(
                    subject_identifier=instance.subject_identifier,
                    report_datetime=instance.consent_datetime,
                    randomizer_name=get_meta_version(),
                    medication_names=[medication.name],
                    site_id=instance.site.id,
                )
            except PrescriptionAlreadyExists:
                pass
            except CommandError as e:
                sys.stdout.write(f"Failed for {instance.subject_identifier}. Got {e}\n")



# Generated by Django 5.2.10 on 2026-01-22 19:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0021_diagnosticdevices"),
        ("meta_subject", "0238_historicalhivexitreview_available_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="glucosefbg",
            name="diagnostic_device",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"name__icontains": "FBG"},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AddField(
            model_name="historicalglucosefbg",
            name="diagnostic_device",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"name__icontains": "FBG"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.diagnosticdevices",
            ),
        ),
    ]



from .ae_followup import AeFollowup
from .ae_initial import AeInitial
from .ae_local_review import AeLocalReview
from .ae_sponsor_review import AeSponsorReview
from .ae_susar import AeSusar
from .ae_tmg import AeTmg
from .death_report import DeathReport
from .death_report_tmg import DeathReportTmg
from .death_report_tmg_second import DeathReportTmgSecond
from .hospitalization import Hospitalization

__all__ = [
    "AeFollowup",
    "AeInitial",
    "AeLocalReview",
    "AeSponsorReview",
    "AeSusar",
    "AeTmg",
    "DeathReport",
    "DeathReportTmg",
    "DeathReportTmgSecond",
    "Hospitalization",
]



# Generated by Django 5.0.1 on 2024-02-07 22:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0054_alter_losstofollowup_options_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endofstudy",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



from .birth_outcomes_form_validator import BirthOutcomesFormValidator
from .delivery_form_validator import DeliveryFormValidator
from .dm_endpoint_form_validator import DmEndpointFormValidator
from .dm_followup_form_validator import DmFollowupFormValidator
from .egfr_drop_notification_form_validator import EgfrDropNotificationFormValidator
from .followup_examination_form_validator import FollowupExaminationFormValidator
from .glucose_fbg_form_validator import GlucoseFbgFormValidator
from .glucose_form_validator import GlucoseFormValidator
from .health_economics_form_validator import HealthEconomicsFormValidator
from .hiv_exit_review_form_validator import HivExitReviewFormValidator

__all__ = [
    "BirthOutcomesFormValidator",
    "DeliveryFormValidator",
    "DmEndpointFormValidator",
    "DmFollowupFormValidator",
    "EgfrDropNotificationFormValidator",
    "FollowupExaminationFormValidator",
    "GlucoseFbgFormValidator",
    "GlucoseFormValidator",
    "HealthEconomicsFormValidator",
    "HivExitReviewFormValidator",
]



# Generated by Django 5.2.6 on 2025-09-30 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_rando", "0007_spfqlist"),
    ]

    operations = [
        migrations.DeleteModel(
            name="SpfqList",
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-24 16:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0038_auto_20200520_0020"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L (millimoles/L)"), ("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L (micromoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L (millimoles/L)"), ("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="fasting_glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (fasting glucose)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="ogtt_two_hr_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="Units (Blood glucose 2hrs after...)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("hospital_notes", "Hospital notes"),
                    ("outpatient_cards", "Outpatient cards"),
                    ("patient", "Patient"),
                    ("collateral_history", "Collateral History from relative/guardian"),
                    ("N/A", "Not applicable (if missed)"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What is the main source of this information?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled visit"),
                    ("unscheduled", "Unscheduled visit"),
                    ("missed", "Missed visit"),
                ],
                help_text="If 'missed', fill in the separate missed visit report",
                max_length=25,
                verbose_name="What is the reason for this visit report?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="info_source",
            field=models.CharField(
                choices=[
                    ("hospital_notes", "Hospital notes"),
                    ("outpatient_cards", "Outpatient cards"),
                    ("patient", "Patient"),
                    ("collateral_history", "Collateral History from relative/guardian"),
                    ("N/A", "Not applicable (if missed)"),
                    ("OTHER", "Other"),
                ],
                max_length=25,
                verbose_name="What is the main source of this information?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled visit"),
                    ("unscheduled", "Unscheduled visit"),
                    ("missed", "Missed visit"),
                ],
                help_text="If 'missed', fill in the separate missed visit report",
                max_length=25,
                verbose_name="What is the reason for this visit report?",
            ),
        ),
    ]



# Generated by Django 5.1 on 2024-10-09 16:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0050_alter_endpoints_created"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="endpoints",
            name="baseline_datetime",
        ),
        migrations.RemoveField(
            model_name="endpoints",
            name="fbg_datetime",
        ),
        migrations.RemoveField(
            model_name="endpoints",
            name="offstudy_datetime",
        ),
        migrations.AddField(
            model_name="endpoints",
            name="baseline_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="endpoints",
            name="fbg_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="endpoints",
            name="offstudy_date",
            field=models.DateField(null=True),
        ),
    ]



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_appointment.constants import MISSED_APPT
from edc_appointment.models import Appointment
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import OffScheduleModelFormMixin

from meta_subject.models import HivExitReview

from ..models import OffSchedule


class OffScheduleFormValidator(PrnFormValidatorMixin, FormValidator):
    pass


class OffScheduleForm(
    OffScheduleModelFormMixin,
    SiteModelFormMixin,
    ActionItemFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = OffScheduleFormValidator
    report_datetime_field_attr = "offschedule_datetime"

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        try:
            HivExitReview.objects.get(
                subject_visit__subject_identifier=self.get_subject_identifier()
            )
        except ObjectDoesNotExist as e:
            # if last visit was missed do not require
            last_appointment = (
                Appointment.objects.filter(
                    subject_identifier=self.get_subject_identifier(),
                    subjectvisit__isnull=False,
                )
                .order_by("appt_datetime")
                .last()
            )
            if last_appointment.appt_timing != MISSED_APPT:
                raise forms.ValidationError(
                    "Hiv Exit Review CRF is required before taking subject off schedule."
                ) from e

    class Meta:
        model = OffSchedule
        fields = "__all__"



from clinicedc_constants import NULL_STRING
from django.db import models
from edc_action_item.models import ActionItem, ActionModelMixin
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_protocol_incident.constants import PROTOCOL_INCIDENT_ACTION
from edc_protocol_incident.model_mixins import ProtocolIncidentModelMixin
from edc_sites.model_mixins import SiteModelMixin

from ..choices import ACTION_REQUIRED


class ProtocolIncident(
    SiteModelMixin,
    ActionModelMixin,
    ProtocolIncidentModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    BaseUuidModel,
):
    action_name = PROTOCOL_INCIDENT_ACTION

    action_item = models.ForeignKey(
        ActionItem,
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        related_name="meta_prn_action_item",
    )

    action_required_old = models.CharField(
        max_length=45, choices=ACTION_REQUIRED, default=NULL_STRING
    )

    def natural_key(self):
        return (self.action_identifier,)

    class Meta(ProtocolIncidentModelMixin.Meta, BaseUuidModel.Meta):
        pass



# Generated by Django 6.0 on 2025-02-01 03:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0217_alter_historicalnextappointment_appt_datetime_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="appt_date",
            field=models.DateField(
                blank=True,
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment date and time",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="appt_date",
            field=models.DateField(
                blank=True,
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="appt_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment date and time",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_prn_admin
from ..forms import DmReferralForm
from ..models import DmReferral


@admin.register(DmReferral, site=meta_prn_admin)
class DmReferralAdmin(
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    ActionItemModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = DmReferralForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        ("Referral to Diabetes clinic", {"fields": ("referral_date",)}),
        (
            "Diabetes diagnosis",
            {
                "fields": ("referral_note",),
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "report_datetime",
        "referral_date_as_col",
    )

    list_filter = (
        "report_datetime",
        "referral_date",
    )

    @admin.display(description="Referral date", ordering="referral_date")
    def referral_date_as_col(self, obj=None):
        return obj.referral_date
